#include "GReATConfigUsage.h"
#include "GreatConfig.h"
#include <Uml.h>

typedef std::set< GR::RewritingRule> RewritingRuleSet;

void GReATConfigUsage::getStartRule( const GR::RootFolder& grRootFolder, GR::RewritingRule& startRule)
{
	std::vector< std::string> startRulePath= _usageInfo.StartRulePath();
	const RewritingRuleSet& rootRules = grRootFolder.RewritingRule_kind_children();
	getStartRule( rootRules, startRulePath, startRule);
}

void GReATConfigUsage::getTemporaryFileUsages( std::vector< GReATConfig::TemporaryFileUsage>& tempFiles)
{
	std::vector< GReATConfig::TemporaryFileUsage> tfus;
	getTemporaryFileUsageKinds( tfus);
	for( std::vector< GReATConfig::TemporaryFileUsage>::const_iterator i= tfus.begin(); i!= tfus.end(); ++i) {
		const GReATConfig::TemporaryFileUsage& currTFU= *i;
		if ( currTFU.type() == GReATConfig::TemporaryFileUsage::meta) {
			tempFiles.push_back( currTFU);
		}
	}
}

void GReATConfigUsage::getTemporaryFileUsageKinds( std::vector< GReATConfig::TemporaryFileUsage>& tempFileKinds)
{
	tempFileKinds= _usageInfo.TemporaryFileUsage_kind_children();
}

void GReATConfigUsage::getFileUsageFileIDs( std::vector< std::string>& fileIDs)
{
	fileIDs.clear();
	const std::set< GReATConfig::FileUsage>& fileUsages= _usageInfo.FileUsage_kind_children();
	for( std::set< GReATConfig::FileUsage>::const_iterator i= fileUsages.begin(); i!= fileUsages.end(); ++i)
	{
		fileIDs.push_back( ( std::string)i->FileID());
	}
}

bool GReATConfigUsage::getMetaName( const std::string& fileID, std::string& metaName)
{
	GReATConfig::FileUsage fileUsage;
	if ( false == getFileUsage( fileID, fileUsage))
		return false;
	metaName= fileUsage.MetaName();
	return true;
}

bool GReATConfigUsage::getMode( const std::string& fileID, std::string& mode)
{
	GReATConfig::FileUsage fileUsage;
	if ( false == getFileUsage( fileID, fileUsage))
		return false;
	mode= fileUsage.Mode();
	return true;
}

bool GReATConfigUsage::getRootClassName( const std::string& fileID, std::string& rootClassName)
{
	GReATConfig::FileUsage fileUsage;
	if ( false == getFileUsage( fileID, fileUsage))
		return false;
	rootClassName= fileUsage.RootClassName();
	return true;
}

bool GReATConfigUsage::getDTDFileName( const std::string& fileID, std::string& dtdFileName)
{
	GReATConfig::FileUsage fileUsage;
	if ( false == getFileUsage( fileID, fileUsage))
		return false;
	dtdFileName= fileUsage.DTDFileName();
	return true;
}

bool GReATConfigUsage::getRunInMemory( const std::string& fileID, bool& runInMemory)
{
	GReATConfig::FileUsage fileUsage;
	if ( false == getFileUsage( fileID, fileUsage))
		return false;
	runInMemory= fileUsage.RunInMemory();
	return true;
}

void GReATConfigUsage::getTransformationInputs( std::set< GReATConfig::TransformationInput>& inputs)
{
	inputs= _usageInfo.TransformationInput_kind_children();
}

bool GReATConfigUsage::parseObjectPathElem( 
		const std::string& pathElemExpr, 
		std::string& classType, 
		std::vector< std::pair< std::string, std::string> >& classAttrs
		)
{
	const std::string ATTR_BEGINBLOCK( "{");
	const std::string ATTR_ENDBLOCK( "}");
	const std::string ATTR_SEP( ",");
	const std::string ATTR_EQ( "=");
	bool isValid= false;
	// start parsing
	std::string::size_type findAttr= pathElemExpr.find( ATTR_BEGINBLOCK);
	if ( findAttr == std::string::npos)
	{
		// no attributes->the entire expression is class type
		classType= pathElemExpr;
		return isValid= true;
	}
	// there are attributes..
	classType.assign( pathElemExpr.begin(), pathElemExpr.begin()+findAttr);
	std::string::size_type findAttrEnd= pathElemExpr.find( ATTR_ENDBLOCK);
	if ( findAttrEnd == std::string::npos)
	{
		// no end-of-attributeblock found
		return isValid= false;
	}
	std::string attrBlock;
	attrBlock.assign( pathElemExpr.begin()+findAttr+1, pathElemExpr.begin()+findAttrEnd);
	if ( false==attrBlock.empty())
		attrBlock+= ATTR_SEP;	// put terminating attribute separator to be able to parse attribute elements in a uniform way
	// iterate over attributes
	std::string::size_type findAttrSep= attrBlock.find( ATTR_SEP);
	while ( findAttrSep != std::string::npos)
	{
		// process attrBlock[0..findAttrSep-1]
		std::string attrElem;
		attrElem.assign( attrBlock.begin(), attrBlock.begin()+ findAttrSep);
		std::string::size_type findAttrEq= attrElem.find( ATTR_EQ);
		if( findAttrEq== std::string::npos)
			return isValid= false;
		classAttrs.push_back( std::make_pair( attrElem.substr( 0, findAttrEq), attrElem.substr( findAttrEq+1, attrElem.size()- findAttrEq-1)));
//		std::cout << attrElem << "\t" << classAttrs.back().first << "= " << classAttrs.back().second << std::endl;
		// continue with attrBlock[findAttrSep+1..size(attrBlock)]
		attrBlock.erase( attrBlock.begin(), attrBlock.begin()+findAttrSep+1);
		findAttrSep= attrBlock.find( ATTR_SEP);
	}
	return isValid;
}

void GReATConfigUsage::getStartRule( const set< GR::RewritingRule>& ruleset, vector< string>& startRulePath, GR::RewritingRule& startRule)
{
	if ( startRulePath.empty())
		return;
	const string& rname = *(startRulePath.begin());
	GR::RewritingRule rule;
	for( RewritingRuleSet::const_iterator i= ruleset.begin(); i!= ruleset.end(); ++i)
	{
		if( ( std::string) i->name()== rname) 
		{
			rule = *i;
			break;
		}
	}
	if( rule== Udm::null) 
		throw udm_exception( ( std::string) rule.name()+ " not exist.");
	startRulePath.erase( startRulePath.begin());
	if( !startRulePath.empty())
	{
		// recursion
		set< GR::RewritingRule> children= rule.RewritingRule_kind_children();
		getStartRule( children, startRulePath, startRule);
	}
	// end of traversal, start rule found.
	startRule = rule;
}

bool GReATConfigUsage::getFileUsage( const std::string& fileID, GReATConfig::FileUsage& fileUsage)
{
	if ( _fileUsageMap.empty())
	{
		// fill map initially.
		const std::set< GReATConfig::FileUsage>& fileUsages= _usageInfo.FileUsage_kind_children();
		for( std::set< GReATConfig::FileUsage>::const_iterator i= fileUsages.begin(); i!= fileUsages.end(); ++i)
		{
			_fileUsageMap[ ( std::string)i->FileID()]= *i;
		}
	}
	// search
	std::map< std::string, GReATConfig::FileUsage>::const_iterator itFind= _fileUsageMap.find( fileID);
	if ( _fileUsageMap.end() == itFind)
		return false;
	fileUsage= itFind->second;
	return true;	
}

void GReATConfigUsage::addFileUsage( const GReATConfig::TemporaryFileUsage& tfu, const std::string fileID, bool runInMem, const std::string& mode)
{
	GReATConfig::FileUsage newFU= GReATConfig::FileUsage::Create( _usageInfo);
	newFU.MetaName()= tfu.MetaName();
	newFU.RootClassName()= tfu.RootClassName();
	newFU.DTDFileName()= tfu.DTDFileName();
	newFU.FileID()= fileID;
	newFU.RunInMemory()= runInMem;
	newFU.Mode()= mode;
}

