#include "GRCGenOps.h"
#include <fstream>
#include "GReATUtils.h"
#include "Uml.h"

using std::endl;
///////////////////////////////////////////////////////////////////////////////
class SpecCmpFuncPrinter {
public:
	SpecCmpFuncPrinter( std::ostream& hos, std::ostream& cos, std::ostream& defos) 
		: _hos( hos), _cos( cos), _defos( defos)
	{}
	void operator()( const std::pair< std::string, std::list< std::string> >& cmpSpec) {
		_funcname= cmpSpec.first;
		for_each_ref( cmpSpec.second.begin(), cmpSpec.second.end(), *this);
	}
	void operator()( const std::string& type) {
		std::string lhsName= "lhs" + convertToCPlusPlusName( type);
		std::string rhsName= "rhs" + convertToCPlusPlusName( type);
		std::string specFuncName= generateCmpSpecName( _funcname, type);
		// cpp file
		_cos << "bool " << specFuncName << "( const Udm::Object& lhs, const Udm::Object& rhs) {" << endl;
		_cos << "\t" << type << " " << lhsName << "= " << type << "::Cast( lhs);" << endl;
		_cos << "\t" << type << " " << rhsName << "= " << type << "::Cast( rhs);" << endl;
		_cos << "\t" << "return " << _funcname << "( " << lhsName << ", " << rhsName << ");" << endl;
		_cos << "}" << endl << endl;
		// header file
		_hos << "extern bool " << specFuncName << "( const Udm::Object& lhs, const Udm::Object& rhs);" << endl;
		// def file
		_defos << specFuncName << endl;
	}
private:
	std::ostream& _hos;
	std::ostream& _cos;
	std::ostream& _defos;
	std::string _funcname;
};

///////////////////////////////////////////////////////////////////////////////
void generateSpecCompareFunctions( const CmpFuncSpecTypes_t& cmpFuncSpecTypes, const std::string& filenamebase) {
	// create files
	std::string GreatTempPath= getGreatTempDir();
	std::ofstream genops_cpp( (GreatTempPath+ filenamebase+ "_genops.cpp").c_str());
	std::ofstream genops_h( (GreatTempPath+ filenamebase+ "_genops.h").c_str());
	std::ofstream gen_def( (GreatTempPath+ filenamebase+ ".def").c_str(), std::ios_base::out | std::ios_base::app);
	// print #include code
	genops_cpp << "#include \"" << filenamebase << "_genops.h\"" << endl << endl;
	genops_h << "#include \"" << filenamebase << "_cmptempl.h\"" << endl << endl; // cmp func templates
	genops_h << "#include \"" << filenamebase << "_inc.h\"" << endl << endl; // UDM domain specific API includes
	// print specialized cmp functions
	SpecCmpFuncPrinter funcPrinter( genops_h, genops_cpp, gen_def);
	for_each_ref( cmpFuncSpecTypes.begin(), cmpFuncSpecTypes.end(), funcPrinter);
	// close files
	genops_cpp.close();
	genops_h.close();
	gen_def.close();
}

///////////////////////////////////////////////////////////////////////////////
void createIncludeMetaFiles( const std::string& fnbase, const std::list< std::string>& metanames) {
	string includefilename= fnbase + "_inc.h";
	std::ofstream includefile;
	includefile.open( includefilename.c_str());
// generate meta includes
	std::list< std::string>::const_iterator it = metanames.begin();
	for(;it != metanames.end(); ++it) {
		includefile << "#include \"" << *it << ".h\"\n";
	}
}

using namespace GR;
///////////////////////////////////////////////////////////////////////////////
void GRCCmpFuncCollector::Visit_RootFolder(const RootFolder& rf)
{
	_specTypes.clear();
// traverse rules
	vector<RewritingRule> rules = rf.RewritingRule_children();
	for(vector<RewritingRule>::iterator itv =rules.begin(); itv!=rules.end(); ++itv) {	
		RewritingRule rule = *itv;
		rule.Accept(*this);
	}
// filter out duplicates, so that one type occurs only once
	for( CmpFuncSpecTypes_t::iterator it= _specTypes.begin(); it!= _specTypes.end(); ++it) {
		std::list< std::string>& types= it->second;
		types.sort();
		types.unique();
	}
}
 
void GRCCmpFuncCollector::Visit_RewritingRule(const RewritingRule& rule)
{
	const string &rtype = rule.ruleType();
// check rule-s and case-s
	if(rtype == "RULE" || rtype == "CASE") {
		std::vector< GR::PatternObject> pos= rule.PatternObject_kind_children();
		for( std::vector< GR::PatternObject>::const_iterator it= pos.begin(); it!= pos.end(); ++it) {
			const std::string& cmpfunc= it->cmpfunc();
			if ( !cmpfunc.empty()) {
				_specTypes[ cmpfunc].push_back( ( string)it->paradigmName() + "::" + ( string)it->classType());
			}
		}
	}
// traverse rules
	const vector<RuleBase> &subrules = rule.RuleBase_kind_children();
	for(vector<RuleBase>::const_iterator i= subrules.begin(); i!= subrules.end(); ++i)
	{	const RuleBase &rb = *i;
		const string &srtype = rb.type().name();
		if( srtype != "RewritingRule" )
			continue;
		RewritingRule subrule = RewritingRule::Cast(rb);
		subrule.Accept(*this);
	}
}