#include "GReATUtils.h"
#include <stdlib.h> // for _ltoa()
#include <locale>
#include<boost/tokenizer.hpp>
#include <Windows.h> // for GetEnvironmentVariable()
#include <direct.h> // for _mkdir()
#include <time.h> 

std::string upperBegin( const std::string& toConvert)
{
	if ( true == toConvert.empty())
		return std::string();
	char* upper= _strupr( strdup( toConvert.c_str()));
	// if toConvert.length == 1, then toConvert.substr copies 0 bytes.
	return std::string( upper).substr( 0, 1) + toConvert.substr( 1, toConvert.length()-1);
}

std::string lowerBegin( const std::string& toConvert)
{
	if ( true == toConvert.empty())
		return std::string();
	char* lower= _strlwr( strdup( toConvert.c_str()));
	// if toConvert.length == 1, then toConvert.substr copies 0 bytes.
	return std::string( lower).substr( 0, 1) + toConvert.substr( 1, toConvert.length()-1);
}

std::string longHexToString( long toConvert)
{
	char temp[ 100];
	_ltoa( toConvert, temp, 16);
	return std::string( temp);
}

std::string convertEscapeSequences( const std::string& toConvert)
{
	std::string escaped= "";
	for( unsigned int i=0; i< toConvert.length(); ++i)
	{
		std::string::const_reference currChar= toConvert[ i];
		if (currChar == static_cast< std::string::value_type>('\\'))
			escaped+= "\\\\";
		else if (currChar == static_cast< std::string::value_type>('\?'))
			escaped+= "\\\?";
		else if (currChar == static_cast< std::string::value_type>('\''))
			escaped+= "\\\'";
		else if (currChar == static_cast< std::string::value_type>('\"'))
			escaped+= "\\\"";
		else
			escaped+= currChar;
	}
	return escaped;
}

char* dropExtension( char* filename)
{
	char* ext= strrchr( filename, '.');
	if( NULL!= ext)
	{
		// 'ext' points to '.' -> 'ext+1' must be valid.
		if ( *(ext+1) != '\\')
			*ext= '\0';	// do not cut filenames with path but without extension, like: "..\x" , ".\x"
	}
	return filename;
}

std::string dropExtension( const std::string& filename)
{
	std::string nameNoExt( filename);
	std::string::size_type findExt= nameNoExt.rfind( '.');
	if ( findExt != std::string::npos)
		if( *(nameNoExt.begin()+ findExt+ 1) != '\\')
			nameNoExt.erase( nameNoExt.begin()+ findExt, nameNoExt.end());	// delete '.' too.
	return nameNoExt;
}

std::string getExtension( const std::string& filename)
{
	std::string ext( filename);
	std::string::size_type findExt= ext.rfind( '.');
	if ( findExt != std::string::npos)
		if( *(ext.begin()+ findExt+ 1) != '\\')
			ext.erase( ext.begin(), ext.begin()+ ++findExt);	// delete '.' too.
		else
			ext.clear();
	return ext;
}

std::string dropPath( const std::string& filepath)
{
	std::string filename( filepath);
	std::string::size_type findPath= filename.rfind( '\\');
	if ( findPath != std::string::npos)
		filename.erase( filename.begin(), filename.begin()+ ++findPath);	// delete '\' too.
	return filename;
}

std::string getPath( const std::string& filepath)
{
	return getPath( filepath, false);
}

std::string getPath( const std::string& filepath, bool deleteTerminatingBackSlash)
{
	std::string filename( filepath);
	std::string::size_type findPath= filename.rfind( '\\');
	if ( findPath != std::string::npos)
		filename.erase( filename.begin()+ ( deleteTerminatingBackSlash ? findPath : ++findPath), filename.end());	// do not delete terminating '\'.
	else
		filename.erase();
	return filename;
}

std::string convertToCPlusPlusName( const std::string& name)
{
	// empty name
	if ( name.empty())
		return "";
	// name must start with letter
	std::string converted;
	if ( false== isalpha( name[ 0]))
		converted= "pre";
	// name can contain alphanum characters and '_'
	for( std::string::const_iterator i= name.begin(); i!= name.end(); ++i)
	{
		if ( isalnum( *i) || (*i== '_'))
			converted+= *i;
		else
			converted+= '_';
	}
	return converted;

}

std::string convertToMpcName( const std::string& name)
{
	// empty name
	if ( name.empty())
		return "";
	std::string converted;
	// Spaces and '-' are translated to '_'.
	for( std::string::const_iterator i= name.begin(); i!= name.end(); ++i)
	{
		if ( (*i== '-') || (*i==' '))
			converted+= '_';
		else
			converted+= *i;
	}
	return converted;
}

int tokenize_csv_lines( const std::string& line, std::vector< std::string>& tokens)
{
	tokens.clear();
	int n_toks= 0;
	boost::tokenizer< boost::escaped_list_separator< char> > 
		tok( 
			line.begin(), 
			line.end(), 
			boost::escaped_list_separator< char>( 0, ',', 0)
			);
	for( boost::tokenizer< boost::escaped_list_separator< char> >::iterator beg=tok.begin(); beg!=tok.end(); ++beg){
		tokens.push_back( *beg);
		++n_toks;
	}
	return n_toks;
}

std::string putPathIntoCmdLine( const std::string& toConvert) {
	if ( toConvert.empty())
		return "";
	std::string converted;
	char lastCh= toConvert[ toConvert.size()-1];
	if (( lastCh == '/') || ( lastCh == '\\')) {
		converted= toConvert.substr( 0, toConvert.size()-1);
	}
	converted.insert( 0, "\"");
	converted+= "\"";
	return converted;
}

std::string generateCmpSpecName( const std::string& genericFuncName, const std::string& type) {
	return convertToCPlusPlusName( genericFuncName + "_of_" + type);
}

std::string getGreatTempDir() {
	std::string greatPath;
	const int BUFSIZE= 500;
	TCHAR tchBuf[ BUFSIZE] = {'\0'}; 
	if ( GetEnvironmentVariable( "GREAT_PATH", tchBuf, BUFSIZE))
	{
		greatPath = std::string(tchBuf);
		greatPath+= "\\tmp\\";
		_mkdir( greatPath.c_str());
	} else {
		return "";
	}
	return greatPath;
}

std::string timestamp( const std::string& filen) {
	time_t ltime;
	time( &ltime );
	return "/* " + filen + " generated on " + ctime( &ltime) + " */";
}

std::string getStlDir() {
#if _MSC_VER >= 1300
	std::string stldir= "";
#else
	std::string stldir= "\"$(UDM_3RDPARTY_PATH)/stl\"";
#endif
	return stldir;
}

std::string getCwd() {
	const int BUFSIZE= 500;
	TCHAR tchBuf[ BUFSIZE] = {'\0'}; 
	_getcwd( tchBuf, BUFSIZE);
	return tchBuf;
}

void changeDir( const std::string& dir) {
	_chdir( dir.c_str());
}
