@echo on
REM	* GReAT_rel.bat <VSS_DB_DIR> <VSS_User> <GReAT_Download_Dir> 
REM	*		<3rdParty_Dir> <3rdParty_Download_Dir> <UDM_Dir> <GReAT_Ver> [-b] [-p]
REM	*
REM	* This batch file creates a GReAT release
REM	*	- get latest source files from a VSS database
REM	*	- get 3rd party files from a specific location
REM	*	- update the version numbers in binaries and readme files
REM	*	- build the binary files
REM	*	- build the release
REM	*
REM	* Arguments:
REM	*	1. <VSS_DB_DIR>: The location of the SRCSAFE.INI file for the GReAT VSS database. 
REM	*	2. <VSS_User>: VSS user name to connect to the GReAT VSS database
REM	*	3. <GReAT_Download_Dir>: The directory where GReAT source files should go to
REM	*	4. <3rdParty_Dir>: The location of the GReAT 3rdParty zip file: 3rdParty_source.zip
REM	*	5. <3rdParty_Download_Dir>: The directory where 3rdParty source files should go to	
REM	*	6. <UDM_Dir>: The directory where UDM was installed
REM	*	7. <GReAT_Ver>: Version number of the GReAT release
REM	*
REM	*  Switches: (specify after arguments!)
REM	*	[-b]: Optional. Fetch&Build mode (don't build release)
REM	* 	[-p]: Optional. Do not download 3rdparty sources
REM	*
REM	* Requirements:
REM	*	- Microsoft Visual Studio 6.0 with SP6
REM	*	- Microsoft Visual SourceSafe 6.0
REM	*	- InstallShield Developer 8.0 with SP1
REM	*	- GME 4.11.10
REM	*	- UDM 2.21
REM	*
REM	* Missing features:
REM	*	1. Adding changelog to releaseinfo.txt and readme.txt
REM	*	2. .doc->.pdf conversion
REM	* 	3 Update UDM version number in setup.rul

REM	* Initialization
	set GREAT_PATH=%3
	set GREAT_LOGF=%GREAT_PATH%\Release\great_rel_log.txt
	cd /d %GREAT_PATH%
	echo ******************************************************************************** >  %GREAT_LOGF%
	echo * GReAT release build process started %DATE% %TIME% >> %GREAT_LOGF%
	echo ******************************************************************************** >>  %GREAT_LOGF%
REM 	* Get Sources from VSS
	echo * SS.exe invoked %DATE% %TIME% >> %GREAT_LOGF%
	call %GREAT_PATH%\Release\fetch_sources.bat %1 %2 $/MobiesTransition/GReAT %3 >>  %GREAT_LOGF%
	IF "%ERRORLEV%" == "1" echo * SS.exe returned 1. This command ran successfully, but your next VSS command may fail. %DATE% %TIME% >> %GREAT_LOGF%
	IF "%ERRORLEV%" == "2" GOTO ErrorHandler
REM	* Unzip 3rd party libraries
	if  "%8" == "-p" GOTO Skip3rdParty
	if  "%9" == "-p" GOTO Skip3rdParty
	%GREAT_PATH%/Release/unzip.exe -u -q -o %4 -d %5
:Skip3rdParty
	set GREAT_3RDPARTY=%5
REM	* Fetch and Build
	if  "%8" == "-b" GOTO BuildBinaries
	if  "%9" == "-b" GOTO BuildBinaries
REM	* Update the version number in the resource files and in GReATVersion.h
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * upgr_bin_ver.bat invoked %DATE% %TIME% >>  %GREAT_LOGF%
	call %GREAT_PATH%\Release\upgr_bin_ver.bat %7	
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * upgr_header_ver.bat invoked %DATE% %TIME% >>  %GREAT_LOGF%
	cd /d %GREAT_PATH%\etc
	ss.exe Checkout $/MobiesTransition/GReAT/etc/GReATVersion.h -I
	call %GREAT_PATH%\Release\upgr_header_ver.bat GReATVersion.h GREAT_VERSION %7
:BuildBinaries
REM	* Build the binaries
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * MSDEV.exe invoked %DATE% %TIME% >>  %GREAT_LOGF%
	set UDM_PATH=%6
	call %GREAT_PATH%\Release\build_bin.bat
	IF "%ERRORLEV%" == "1" GOTO ErrorHandler
REM	* Fetch and Build
	if  "%8" == "-b" GOTO BuildOnlyHandler
	if  "%9" == "-b" GOTO BuildOnlyHandler
REM	* Do the versioning of the release, add changelog
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * is_ver.vbs invoked %DATE% %TIME% >>  %GREAT_LOGF%
	cd /d %GREAT_PATH%\InstallShield
	ss.exe Checkout $/MobiesTransition/GReAT/InstallShield/GREAT_UDM_basic.ism -I
	cd /d %GREAT_PATH%\InstallShield\GREAT_UDM_basic\
	ss.exe Checkout $/MobiesTransition/GReAT/InstallShield/GREAT_UDM_basic/setup.obs -I
	ss.exe Checkout $/MobiesTransition/GReAT/InstallShield/GREAT_UDM_basic/setup.rul -I
	REM
	call %GREAT_PATH%\Release\is_ver.vbs "%GREAT_PATH%\InstallShield\GReAT_UDM_basic.ism" %7
	REM  ver.txt, readme.txt
	cd /d %GREAT_PATH%
	ss.exe Checkout $/MobiesTransition/GReAT/ver.txt -I
	call %GREAT_PATH%\Release\create_ver_txt.bat ver.txt %7
	REM ...
	REM changelog
	REM ...
REM	* Convert .xme files to .mga and to .xmp
	call %GREAT_PATH%\Release\CreateMgaXmpFiles.bat
REM	* Build the release
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * is_path.vbs invoked %DATE% %TIME% >>  %GREAT_LOGF%
	call %GREAT_PATH%\Release\is_path.vbs "%GREAT_PATH%\InstallShield\GReAT_UDM_basic.ism" "%GREAT_PATH%" "%UDM_PATH%\Release"
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * is_build.bat invoked %DATE% %TIME% >>  %GREAT_LOGF%
	call %GREAT_PATH%\Release\is_build.bat "%GREAT_PATH%\InstallShield\GREAT_UDM_basic.ism" "%GREAT_PATH%\InstallShield\\GREAT_UDM_basic\setup.rul"
	GOTO End

:ErrorHandler
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * great_rel.bat FAILED at %DATE% %TIME% >>  %GREAT_LOGF%
	GOTO End
	
:BuildOnlyHandler
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * great_rel.bat Fetch and Build mode finished at %DATE% %TIME% >>  %GREAT_LOGF%
	
:End
	echo ******************************************************************************** >>  %GREAT_LOGF%
	echo * GReAT release build process ended %DATE% %TIME% >> %GREAT_LOGF%
