#include "GeneralBackend.h"

void GeneralBackend::addBackend( const std::string& name, Udm::DataNetwork& dn)
{
	_backendMap[ name]= &dn;
}

void GeneralBackend::setCrossLinkBackend( const std::string& name, Udm::DataNetwork& dn)
{
	_xlinkDiagName= name;
	addBackend( name, dn);
}

const Udm::DataNetwork* GeneralBackend::getBackend( const std::string& name) const
{
	BackendMap_t::const_iterator itFind= _backendMap.find( name);
	return ( itFind == _backendMap.end()) ? NULL : ( *itFind).second;
}

const Uml::Diagram GeneralBackend::getDiagram( const std::string& name) const throw (CGEx)
{
	const Udm::DataNetwork* backend= getBackend( name);
	if ( NULL== backend)
		throw CGEx("FATAL: Backend: "+ name+ " does not exist.");
	return Uml::Diagram::Cast( backend->GetRootObject());
}

const Uml::Diagram GeneralBackend::getCrossLinkDiagram() const throw (CGEx)
{
	return getDiagram( _xlinkDiagName);
}

GeneralBackend& GeneralBackend::operator=( const BackendMap_t& dns)
{
	_backendMap= dns;
	return *this;
}

std::pair< std::string, GlobalDefinition*> createGlobalVariable( const std::string& forType) {
	GlobalDefinition* globalVar= new GlobalDefinition;
	globalVar->_type= forType;
	globalVar->_variable= convertToCPlusPlusName( "g_"+ forType);
	return std::make_pair( forType, globalVar);
}

void GeneralBackend::generateTempRoots( const std::list< std::string>& forRootTypes)
{
	std::transform( forRootTypes.begin(), forRootTypes.end(), std::inserter( _tempRoots, _tempRoots.begin()), &createGlobalVariable);
	int sz= _tempRoots.size();
}

GlobalDefinition* GeneralBackend::getTempRootDef( const std::string& key) const
{
	TempRoots_t::const_iterator itFind= _tempRoots.find( key);
	return ( itFind == _tempRoots.end()) ? 0 : itFind->second;
}
