#include "OutputAppender.h"
#include "PEStringTable.h"
#include "GReATUtils.h"
#include <algorithm>


void OutputAppender::addOutputs( const ParamDatas_t& ruleOutputs, Function& filterFuncDef, FunctionCall& filterFuncCall)
{
	// For every output...
	for( ParamDatas_t::const_iterator itOutputs= ruleOutputs.begin(); itOutputs!= ruleOutputs.end(); ++itOutputs)
	{
		// Do multiple things in parallel...
		std::map< GR::ObjectWrapper, std::string>::const_iterator itBoundObject= _boundObjects.find( itOutputs->_param);
		if ( _boundObjects.end()== itBoundObject)
		{
			throw CGEx("FATAL: CG internal error. Output parameter: "+ ( std::string)itOutputs->_param.name()+ " not bound in output filter.");
		}
		// 1, Add parameter name to function caller.
		const std::string& callerParamName= (*itBoundObject).second;
		filterFuncCall._params.push_back( callerParamName);
		// 2, Add parameter definition to function definition.
		FuncParam_t calleeParam;
		createParam( *itOutputs, calleeParam);
		const std::string& paramName= calleeParam._param.second;
		filterFuncDef.insertInputParam( calleeParam);
		// 3, Create output appender expression into function definition.
		Expression* appendOutput= new Expression;
		const string& outputMemberName= itOutputs->_memberDef->_variable;
		appendOutput->_expression= outputMemberName+ ACC+ "push_back("+ SEP+ lowerBegin( paramName)+ ")";
		filterFuncDef.add( *appendOutput);
	}
}

void OutputAppender::createParam( const ParamData& output, FuncParam_t& param) const
{
	std::set< GR::ObjectWrapper> OWs= _rule.ObjectWrapper_kind_children();
	std::set< GR::ObjectWrapper>::const_iterator itFindOutput= std::find( OWs.begin(), OWs.end(), output._param);
	if ( OWs.end()== itFindOutput)
		throw CGEx("FATAL: Object wrapper belonging to output parameter: "+ output._name+ " not found in rule: "+ ( std::string)_rule.name()+ ".");
	param._param= make_pair(
		CST+ SEP+ ( std::string)itFindOutput->paradigmName()+ RES+ ( std::string)itFindOutput->classType()+ REF, 	// type
		lowerBegin( PE::createUniqueName( ( std::string)itFindOutput->name()))	// name
		);
}