#pragma warning(disable : 4786) 
#include "../GRDebugInterface/GRDebugInterface.h"
#include "GRDBackEnd.h"
#include <iostream>
#include <sstream>
#include <fstream>

namespace GRDI
{


//=========================================
// =========== IGRDBackEnd
//=========================================
//=========================================
bool GRDBackEnd::synchronize()
{
  {
    Lock lkWatch(m_mutexWatch);
  
    if (m_eventSet.size())
    {
      (*m_eventHook)(m_eventSet);
      resetEventSet();
    }
  }
  
  Lock lk(m_mutexSuspend);
   while (m_suspended)
      m_resume.wait(lk);

  resetFlags();
   
  return m_synchStatus != 0;
}
//=========================================

IGRDBackEnd::SynchType GRDBackEnd::getSynchStatus()
{
  return m_synchStatus;
}

//=========================================

const std::string& GRDBackEnd::getProgramFileName()
{
  Lock lkWatch(m_mutexWatch);

  return m_programFileName;
}
//=========================================
const std::string& GRDBackEnd::getWD()
{
  Lock lkWatch(m_mutexWatch);

  return m_workingDirectory;
}
//=========================================
void GRDBackEnd::stepIn(const FunctionCallDescr& funcCallDescr)
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);

  popContext();

  LocationID id = findBeginLocation(funcCallDescr.m_func);

  addNewContext(funcCallDescr, id);

  checkBreakpoints(id);
  
  Event ev;
  ev.m_kind = EVENT_FUNCTION_ENTRY;
  getCurrentContextID(ev.m_u.m_ctx.m_ctxID);
  ev.m_u.m_ctx.m_loc = id;
  m_eventSet.push_back(ev);

  ev.m_kind = EVENT_FUNCTION_STEP;
  ev.m_u.m_singleStep.m_loc = id;
  m_eventSet.push_back(ev);

}

//=========================================
void GRDBackEnd::step(const FunctionCallDescr& funcCallDescr)
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);

  ContextDescr& ctx = popContext();

  LocationID id = findCallLocation(funcCallDescr.m_func);

  //refreshCurrContext(funcCallDescr, id );

  checkBreakpoints(id);
  
  Event ev;
  ev.m_kind = EVENT_FUNCTION_STEP;
  ev.m_u.m_singleStep.m_loc = id;
  m_eventSet.push_back(ev);

}

//=========================================
void GRDBackEnd::stepOut(const FunctionCallDescr& funcCallDescr)
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);

  popContext();

  
  LocationID id = findEndLocation(funcCallDescr.m_func);

  refreshCurrContext(funcCallDescr, id);

  m_popCtxNum++;

  checkBreakpoints(id);
  
  Event ev;
  ev.m_kind = EVENT_FUNCTION_EXIT;
  getCurrentContextID(ev.m_u.m_ctx.m_ctxID);
  ev.m_u.m_ctx.m_loc = id;
  m_eventSet.push_back(ev);

  ev.m_kind = EVENT_FUNCTION_STEP;
  ev.m_u.m_singleStep.m_loc = id;
  m_eventSet.push_back(ev);

}

//=========================================
void GRDBackEnd::setStartFunction(const IFunctionPtr& func)
{
  Lock lk(m_mutexWatch);
  m_startFunc = func;
}
//=========================================
void GRDBackEnd::addRootFunction(const IFunctionPtr& func)
{
   Lock lk(m_mutexWatch);
   m_rootFuncs.insert(func);
}
//=========================================
LocationID GRDBackEnd::addLocation(const LocationID& parLocID, const Location& loc)
{
  Lock lk(m_mutexWatch);
  LocationID currLocID = m_locID;
  Location currLoc = loc;
  currLoc.m_locID = currLocID;
  currLoc.m_parID = parLocID;

  m_id2loc[currLocID] = currLoc;
  const FunctionIDLocationTypePair& pair = dynamic_cast<const FunctionIDLocationTypePair&>( currLoc);
  m_loc2id[pair] = currLocID;

  m_locID++;

  return currLocID;
}

//=========================================
void GRDBackEnd::registerFunctionCall(const FunctionDescr& funcDescr)
{
  Lock lk(m_mutexWatch);
  if (m_functions.find(funcDescr.m_func) != m_functions.end())
    return;

  
  if (m_startFunc == nullFunction)
    throw std::invalid_argument("invalid root function");

  m_functions[funcDescr.m_func] = funcDescr;
}
//=========================================
void GRDBackEnd::saved()
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);

  Event ev;
  ev.m_kind = EVENT_GR_SAVE;
  m_eventSet.push_back(ev);
  m_synchStatus &= ~SAVE;
}

//=========================================
void GRDBackEnd::loaded()
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);


  orderCalls();

  m_locID = 0;
  
  m_rootLocs.clear();

  IFunctionPtrSet::iterator itS = m_rootFuncs.find(m_startFunc);
  if (itS == m_rootFuncs.end())
    throw std::invalid_argument("invalid start function");

  m_rootLocs.push_back(addLocation(invalidLocationID, 
                                Location (nullFunction, m_startFunc, FunctionCall)));

  setLocations(m_rootLocs.front(), m_startFunc);

  m_rootFuncs.erase(itS);

  IFunctionPtrSet::const_iterator it = m_rootFuncs.begin();
  for(; it != m_rootFuncs.end(); ++it)
  {
    
    const IFunctionPtr& func = *it;
    LocationID locID = addLocation(invalidLocationID, 
                                Location (nullFunction, func, FunctionCall));
    m_rootLocs.push_back(locID );
    setLocations(locID, func);
  }

  LocationIDIFunctionPtrMap::const_iterator itR =  m_refMap.begin();
  for(;itR!= m_refMap.end(); ++itR)
  {
    LocIDLocMap::iterator itL = m_id2loc.find(itR->first);
    if (itL == m_id2loc.end())
     throw std::invalid_argument("invalid location");

    Location& loc =  itL->second;
    loc.m_refID = findCallLocation(itR->second); 
  }
  m_refMap.clear();

  Event ev;
  ev.m_kind = EVENT_GR_LOAD;
  m_eventSet.push_back(ev);
 
  m_synchStatus &= ~LOAD;
}
//=========================================
void GRDBackEnd::closed()
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);

  Event ev;
  ev.m_kind = EVENT_GR_CLOSE;
  m_eventSet.push_back(ev);
  clearSource();
  m_synchStatus = 0;
}
//=========================================
void GRDBackEnd::stopped()
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);
  resetContext();

  Event ev;
  ev.m_kind = EVENT_GR_STOPPED;
  m_eventSet.push_back(ev);
  m_synchStatus &= ~STOP;
  
}
//=========================================
void GRDBackEnd::exception(const std::string& str)
{
  Lock lk(m_mutexCtrl);


  m_exceptionDescr =  str;

  Event ev;
  ev.m_kind = EVENT_EXCEPTION;

  ContextID ctxID;
  if (!getCurrentContextID(ctxID))
  {
    getCurrentLocation(ctxID, ev.m_u.m_exception.m_loc);
  }
  else
    ev.m_u.m_exception.m_loc = invalidLocationID;

  m_eventSet.clear();
  m_eventSet.push_back(ev);
  m_synchStatus = 0;
}
//=========================================
void GRDBackEnd::up()
{
  Lock lk(m_mutexCtrl);
  Event ev;
  ev.m_kind = EVENT_GR_ENGINE_UP;
  m_eventSet.push_back(ev);

}
//=========================================
void GRDBackEnd::down()
{
  Lock lk(m_mutexCtrl);
  m_eventSet.clear();
  Event ev;
  ev.m_kind = EVENT_GR_ENGINE_DOWN;
  m_eventSet.push_back(ev);
}
//=========================================
void GRDBackEnd::start()
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);

  resetContext();

  Event ev;
  ev.m_kind = EVENT_GR_START;
  m_eventSet.push_back(ev);

}

//=========================================
void GRDBackEnd::finish()
{
  Lock lk(m_mutexCtrl);
  Lock lkWatch(m_mutexWatch);
  resetContext();

  Event ev;
  ev.m_kind = EVENT_GR_FINISH;
  m_eventSet.push_back(ev);

  m_synchStatus &= ~RUN;

}
//=========================================
void GRDBackEnd::reportProgress(double prg)
{
  Lock lk(m_mutexCtrl);

  Event ev;  
  ev.m_kind = EVENT_GR_PROGRESS;
  ev.m_u.m_progress.m_prgs = prg;
  m_eventSet.push_back(ev);

}
//=========================================
DebugInterface* GRDBackEnd::getDebugInterface()
{
  return (DebugInterface*)this;
}
};//namespace GRDI