
#include <sstream>
#include "GReATDebugger.h"
#include "GReATDebuggerVars.h"
#include "../GRDBackEnd/IGRDBackend.h"


#include <fstream>

//================== GReAT 
//=========================
void GReATDebugger::reportProgress(double prg)
{
  if (m_back)
    m_back->reportProgress(prg);
}
//=========================
bool GReATDebugger::quit()
{
  if (m_back)
    return (m_back->getSynchStatus() != GRDI::IGRDBackEnd::RUN);
  else
    return m_quitNonDebug;
}
//================================
void GReATDebugger::resetTest(const GRDI::FunctionDescr& descr,
                              const GR::RuleBase& rule)
{

  std::string str(GR::RewritingRule::Cast(rule).ruleType()); 

  if ( str  == "TEST")
  {
    GRDI::FunctionTable::const_iterator itC = descr.m_calls.begin();
   for(; itC != descr.m_calls.end(); ++itC)
     (*itC)->getIFunctionImpl()->m_match = -1;
  }
}

//================================
void GReATDebugger::setCase(const GR::RuleBase& rule, 
                            const PassAlongPackets& arguments,
                            GRDI::FunctionCallDescr& funcCallDescr)
{
  std::string str(GR::RewritingRule::Cast(rule).ruleType()); 

  if (str == "CASE")
  {
    int& match = funcCallDescr.m_func->getIFunctionImpl()->m_match; 
    if (arguments.size())
      match =  arguments.front().size();
    else
      match = 0;
  }
}

//================================
void GReATDebugger::stepIn(const GR::RuleBase& rule, const PassAlongPackets& arguments)
{


  if (!quit() && m_back->debugInfo())
  {
    checkInterActions();
    const GRDI::FunctionDescr& funcDescr = findFunction(rule);

    GRDI::FunctionCallDescr funcCallDescr;
    funcCallDescr.m_func = funcDescr.m_func;
    funcCallDescr.m_varValMap = funcDescr.m_varTable;

    addVarValue(funcCallDescr, rule, arguments);

    resetTest(funcDescr, rule);

    m_back->stepIn(funcCallDescr);
  }
  m_back->synchronize();
}

//================================
void GReATDebugger::stepOut(const GR::RuleBase& rule, 
                            const PassAlongPackets& arguments,
                            const MatchVector& match)
{
  
  if (!quit() && m_back->debugInfo())  
  {
    checkInterActions();

    GRDI::FunctionDescr& funcDescr = findFunction(rule);

    GRDI::FunctionCallDescr funcCallDescr;
    funcCallDescr.m_func = funcDescr.m_func;
    funcCallDescr.m_varValMap = funcDescr.m_varTable;

    PassAlongPackets new_arguments(arguments);
    m_great->RemoveDuplicates(new_arguments);
    addVarValue(funcCallDescr, rule, new_arguments);
    addMatchVarValue(funcCallDescr, match);

    setCase(rule, arguments, funcCallDescr);

    m_back->stepOut(funcCallDescr);
  }
  m_back->synchronize();

}


//================================
void GReATDebugger::step(const GR::RuleBase& caller, 
                       const GR::RuleBase& called,
                       const PassAlongPackets& arguments)
{
  if (!quit() && m_back->debugInfo())  
  {
    checkInterActions();

    GRDI::FunctionCallDescr childFuncCallDescr;


    childFuncCallDescr.m_func = findFunction(called).m_func;
    GRDI::FunctionDescr&  funcDescr = findFunction(caller);
    childFuncCallDescr.m_caller = funcDescr.m_func;
    childFuncCallDescr.m_varValMap = funcDescr.m_varTable;
    addVarValue(childFuncCallDescr, caller, arguments);

    m_back->step(childFuncCallDescr);
  }
  m_back->synchronize();
}
