#include "GReATFactory.h"
#include "GReAT.h"
#include "GReATParametersFactory.h"
//#include "CINTCodeGathererFactory.h"

#include <time.h>
#include <direct.h>
#include <conio.h>

#include "GReATVersion.h"
#include "GRCC.h"
#include "GRCNames.h"
#include "GRCLoader.h"
#include "GReATConfig.h"
#include "GReATConfigMeta.h"
#include "GReATParameters.h"
#include "GReATUtils.h"
#include <algorithm>

//============================================
class Notify : public IGReAT::ICtrlNotify
{
public:
  static bool m_quitApp;
public:

  // IGReAT::ICtrlNotify
  void reportProgress(double prg) {}
  bool quit()
  {
    return m_quitApp;
  }
};
//============================================
bool Notify::m_quitApp = false;
//============================================
BOOL WINAPI ConsoleHandler(DWORD CEvent)
{
  bool exit = false;
  switch(CEvent)
  {
    case CTRL_C_EVENT: exit = true; break;
    case CTRL_BREAK_EVENT: exit = true; break;
    case CTRL_CLOSE_EVENT: exit = true; break;
  }
  if (exit)
  {
    Notify::m_quitApp = true;
    return TRUE;
  }
  return FALSE;
}
//============================================
void printVer()
{
	cout << "GR Engine v"<<GREAT_VERSION<<".\nCopyright (c) 2000-2004 ISIS, Vanderbilt University"<<endl<<endl;
}

void usage()
{
	cout << "Usage: GRE.exe <ConfigFileName> [<fileSpec>] ... [-d|-dv]"<<endl;
	cout << "ConfigFileName\tGReAT Configuration information file"<<endl;
	cout << "fileSpec\tRun time file information"<<endl;
	cout <<	"[-d]\tPrint out executing transformation rule name"<<endl;
	cout <<	"[-dv]\tPrint out executing transformation rule name, plus input packets, pattern matches, output packets information"<<endl;
	cout << endl;
}

extern HINSTANCE hCINTDLL;
typedef Udm::UdmDiagram* (*INIT_FN)();
typedef Udm::UdmDiagram* (*MAP_FN)();

void main(int argc, char* argv[])
{
	printVer();
	if (argc <= 1)
	{
		usage();
		return;
	}

  if (SetConsoleCtrlHandler(
      (PHANDLER_ROUTINE)ConsoleHandler,TRUE)==FALSE)
  {
      // unable to install handler... 
      // display message to the user
    std::cout << "Unable to install handler!\n";

  }

	char cwd[_MAX_PATH];
	_getcwd(cwd, _MAX_PATH);

	IGReATParameters* grParams = 0;
	IGReAT* great = 0;
//	ICINTCodeGatherer* cintDll = 0;
#ifndef _DEBUG
	try
	{
#endif
    printf("\nEngine Started. Loading relevent files... \n");

		grParams = GReATParametersFactory::create();
		grParams->setCurrentWorkingPath((string)cwd);

		vector<string> files;
		for(int i=2; i<argc; ++i)
		{
			if(_stricmp(argv[i],"-d") == 0)
				GReATOptions::instance()->printRule = true;
			else if(_stricmp(argv[i],"-dv") == 0)
			{
				GReATOptions::instance()->printRule = true;
				GReATOptions::instance()->printDetail = true;
			}
			else if(_stricmp(argv[i],"-c") == 0)
			{
				grParams->useMemoryBackend();
			}
			else 
				files.push_back(argv[i]);
		}
		if(files.empty()) 
			grParams->open(argv[1]);
		else 
			grParams->open(argv[1], files);

		GeneralBackendPtr dataNetworkMap;
		grParams->getGeneralBackend(dataNetworkMap);

		UdmDataNetworkPtr transformation;
		grParams->getTransformation(transformation);

		great = GReATFactory::create();
		great->init(dataNetworkMap, transformation);

    Notify notify;
   great->setCtrlNotify(&notify);

///////////////////////////////////////////////////////////////////////////////
/*
    cintDll = CINTCodeGathererFactory::create();
    UserCodeGenConfig cfg;
    cfg.m_depFileName = grParams->getTransfLastWrittenFileName();
    cfg.m_udmDirName = grParams->getUdmDirName();
    cfg.m_transf = transformation;
    cfg.m_metas = grParams->getMetas();

//    StrVec incFiles, libFiles, incDirs, libDirs;
//    grParams->getUserCodeInfo(incFiles, libFiles, incDirs, libDirs);

//    cfg.m_incFiles = incFiles;
//    cfg.m_libFiles = libFiles;
//    cfg.m_incDirs = incDirs;
//    cfg.m_libDirs = libDirs;

    #ifdef _DEBUG
      cfg.m_debug = true;
    #else
      cfg.m_debug = false;
    #endif

    cintDll->generateDll(cfg);
    HINSTANCE hDll  = cintDll->loadDll();
    great->setCINTDll(hDll);
*/
///////////////////////////////////////////////////////////////////////////////

	GReATConfig::RootFolder cfgRoot= GReATConfig::RootFolder::Cast( ((GReATParameters*)grParams)->sdnConfig->GetRootObject());
	std::set< GReATConfig::MetaInformation> mis= cfgRoot.MetaInformation_kind_children();	
	GReATConfigMeta metaInfo( *mis.begin());
	std::string configPath = getPath( argv[1]);
	if(configPath.empty()) configPath = ".\\";
	configPath = GReATConfigMeta::getFullPath((string)cwd, configPath);

	CCodeGenConfig cc_cfg;
	cc_cfg._transFileName= GReATConfigMeta::getFullPath( configPath, metaInfo.getTransformationFileName());
	cc_cfg._depFileName= cc_cfg._transFileName;
	cc_cfg._udmDirName= getPath( GReATConfigMeta::getFullPath( configPath, metaInfo.getUdmProjectFileName()), false);
	//
	typedef std::map< std::string, const Udm::DataNetwork*> BackendMap_t;
	BackendMap_t dns;
	metaInfo.getGeneralBackend( dns, configPath);
	typedef std::list< std::string> StrList_t;
	StrList_t metanames;
	getFirst( std::make_pair( std::string(), (const Udm::DataNetwork*) 0));	// template instantiation to satisfy the linker
	std::transform( dns.begin(), dns.end(), std::back_inserter( metanames), getFirst< std::string, const Udm::DataNetwork*>);
	if ( metaInfo.isUdmProjectFile()) {
		metanames.push_back( metaInfo.getCrossLinkDiagramName());
	}
	cc_cfg._metas= metanames;
	//
    #ifdef _DEBUG
	    cc_cfg._debug = true;
    #else
      cc_cfg._debug = false;
    #endif
	cc_cfg._dynamic= true;
	//
	GRCC grcc( cc_cfg);
	std::cout << "Start generation. config path= " << configPath << endl;
	grcc.generateLibrary();
	//
	std::cout << "Start load dll." << endl;
	std::string dll_name= getGreatTempDir() + GRCNames::generateDllName( dropPath( dropExtension( cc_cfg._transFileName)),cc_cfg._debug);
	GRCLoader grcLoader( dll_name, grParams->getMetas());
    HINSTANCE hDll  = grcLoader.loadDll();
    great->setCINTDll(hDll);


		GR::RewritingRule startRule;
		PassAlongPackets arguments;
		grParams->getStartRule(startRule);
		grParams->getPassAlongPackets(arguments);

    time_t   start, finish;
    if (!Notify::m_quitApp)
    {
		  start=clock();
      cout << "Running...\n" <<std::endl;    
		  great->Execute(startRule, arguments);

    }
    if (Notify::m_quitApp)
    {
      cout << "The execution has been quitted.\n" <<std::endl;
      grParams->closeNoSave();
    }
    else
    {
		  finish = clock();
		  double duration = (double)(finish - start) / CLOCKS_PER_SEC;
		  if(GReATOptions::instance()->printDetail || GReATOptions::instance()->printRule) 
			  cout << "\n The Transformation took " << duration << "seconds" <<endl;
		  
    //====================
    // the way of restarting GR without loading rewriting rules
/*      grParams->saveProgram();
 	  grParams->closeProgram();
		grParams->reloadProgram();
   
     grParams->getGeneralBackend(dataNetworkMap);
     great->init(dataNetworkMap, transformation);
 
     grParams->getStartRule(startRule);
     grParams->getPassAlongPackets(arguments);
     great->Execute(startRule, arguments);
*/
    }
  //====================
		arguments.clear();
		if(great)
			delete great, great=0;
		grParams->close();
		if(grParams)
			delete grParams, grParams=0;
		grcLoader.closeDll();
//		if (cintDll)
//			delete cintDll, cintDll = 0;

		if (!Notify::m_quitApp) cout<<"Done!"<<endl;
#ifndef _DEBUG
	}
	catch(udm_exception &exc)
	{
		cout<<"Exception: "<<exc.what()<<endl;
		if(great)
			delete great, great=0;	
		if(grParams)
		{
			grParams->closeNoSave();
			delete grParams, grParams=0;
		}
//		if (cintDll)
//			delete cintDll, cintDll = 0;
	}
#endif
 // printf( "Press any key to exit ...\n" );
 // _getch();
}
