// LegacyUpdateVisitor.cpp: implementation of the LegacyUpdateVisitor class.
//
//////////////////////////////////////////////////////////////////////

#include "LegacyUpdateVisitor.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LegacyUpdateVisitor::LegacyUpdateVisitor()
{
}

LegacyUpdateVisitor::~LegacyUpdateVisitor()
{
}

void LegacyUpdateVisitor::Visit_RootFolder(const RootFolder& rf)
{
	{	vector<MetaGME::SheetFolder> sfs = rf.SheetFolder_children();
		for(vector<MetaGME::SheetFolder>::iterator i=sfs.begin(); i!=sfs.end(); i++)
		{	MetaGME::SheetFolder sf = *i;
			sf.Accept(*this);
		}
	}

	{	vector<MetaGME::ParadigmSheet> pas = rf.ParadigmSheet_children();
		for(vector<MetaGME::ParadigmSheet>::iterator i=pas.begin(); i!=pas.end(); i++)
		{	MetaGME::ParadigmSheet pa = *i;
			pa.Accept(*this);
		}
	}
}

void LegacyUpdateVisitor::Visit_SheetFolder(const SheetFolder& sf)
{
	vector<MetaGME::ParadigmSheet> pas = sf.ParadigmSheet_children();
	for(vector<MetaGME::ParadigmSheet>::iterator i=pas.begin(); i!=pas.end(); i++)
	{	MetaGME::ParadigmSheet pa = *i;
		pa.Accept(*this);
	}
}

void LegacyUpdateVisitor::Visit_ParadigmSheet(const ParadigmSheet& pa)
{
	{	vector<MetaGME::Connection> cons = pa.Connection_kind_children();
		for(vector<MetaGME::Connection>::iterator i=cons.begin(); i!=cons.end(); i++)
		{	MetaGME::Connection con = *i;
			con.Accept(*this);
		}
	}

	{	vector<MetaGME::Set> sets = pa.Set_kind_children();
		for(vector<MetaGME::Set>::iterator i=sets.begin(); i!=sets.end(); i++)
		{	MetaGME::Set set = *i;
			set.Accept(*this);
		}
	}

	{	vector<MetaGME::Reference> refs = pa.Reference_kind_children();
		for(vector<MetaGME::Reference>::iterator i=refs.begin(); i!=refs.end(); i++)
		{	MetaGME::Reference ref = *i;
			ref.Accept(*this);
		}
	}
}

void LegacyUpdateVisitor::Visit_Connection(const Connection& con)
{
	MetaGME::Connection connection = con;
	strmap prefMap;
	ConverGeneralPreferenceToMap((string)connection.GeneralPreferences(),prefMap);
	
	string srvalue = GetValueForName("sName",prefMap);
	if(srvalue == "")
	{	srvalue = "src";
		try
		{	connection.GeneralPreferences() = (string)connection.GeneralPreferences() + "\nsName = " + srvalue;
		}
		catch(udm_exception ce) 
		{	//Error
			string error = "Error: Connection:"+(string)connection.name()+ " is read only. The object may be part of a library. Please run this interpreter on the library mga file, refresh library in this file and then run the interpreter";
			throw udm_exception(error);
		}
	}

	string drvalue = GetValueForName("dName",prefMap);
	if(drvalue == "")
	{	drvalue = "dst";
		try
		{	connection.GeneralPreferences() = (string)connection.GeneralPreferences() + "\ndName = " + drvalue;
		}
		catch(udm_exception ce) 
		{	//Error
			string error = "Error: Connection:"+(string)connection.name()+ " is read only. The object may be part of a library. Please run this interpreter on the library mga file, refresh library in this file and then run the interpreter";
			throw udm_exception(error);
		}
	}
}

void LegacyUpdateVisitor::Visit_Reference(const Reference& ref)
{
	MetaGME::Reference reference = ref;
	strmap prefMap;
	ConverGeneralPreferenceToMap((string)reference.GeneralPreferences(),prefMap);
	
	string value;
	
	value = GetValueForName("rName",prefMap);
	if(value == "")
	{	value = "ref";
		try
		{	reference.GeneralPreferences() = (string)reference.GeneralPreferences() + "\nrName = " + value;
		}
		catch(udm_exception ce) 
		{	//Error
			string error = "Error: Reference:"+(string)reference.name() + " is read only. The object may be part of a library. Please run this interpreter on the library mga file, refresh library in this file and then run the interpreter";
			throw udm_exception(error);
		}
	}

	value = GetValueForName("rrName",prefMap);
	if(value == "")
	{	value = "src";
		try
		{	reference.GeneralPreferences() = (string)reference.GeneralPreferences() + "\nrrName = " + value;
		}
		catch(udm_exception ce) 
		{	//Error
			string error = "Error: Reference:"+(string)reference.name() + " is read only. The object may be part of a library. Please run this interpreter on the library mga file, refresh library in this file and then run the interpreter";
			throw udm_exception(error);

		}
	}
}

void LegacyUpdateVisitor::Visit_Set(const Set& setaggr)
{
	MetaGME::Set set = setaggr;
	strmap prefMap;
	ConverGeneralPreferenceToMap((string)set.GeneralPreferences(),prefMap);

	string value;
	value = GetValueForName("mName",prefMap);
	if(value == "")
	{	value = "members";
		try
		{	set.GeneralPreferences() = (string)set.GeneralPreferences() + "\nmName = " + value;
		}
		catch(udm_exception ce) 
		{	//Error
			string error = "Error: Set:"+(string)set.name() + " is read only. The object may be part of a library. Please run this interpreter on the library mga file, refresh library in this file and then run the interpreter";
			throw udm_exception(error);
		}
	}
		
	/*value = GetValueForName("sName",prefMap);
	if(value == "")
	{	value = "src";
		try
		{	set.GeneralPreferences() = (string)set.GeneralPreferences() + "\nsName = " + value;
		}
		catch(udm_exception ce) 
		{	//Error
			MetaGME::ParadigmSheet ps = MetaGME::ParadigmSheet::Cast(set.parent());
			string error = "Error: Set:"+(string)set.name()+ " in ParadigmSheet:" + (string)ps.name() + " is read only. The object may be part of a library. Please run this interpreter on the library mga file, refresh library in this file and then run the interpreter";
			throw udm_exception(error);
		}
	}*/
}

void LegacyUpdateVisitor::ConverGeneralPreferenceToMap(string pref, strmap &prefmap)
{	StringTokenizer stok(pref, " \n,=;");
	StringTokenizer::iterator i,j;
	for(i=stok.begin();i!=stok.end();i++)
	{	j=i;
		i++;
		if(i!=stok.end())
		{	prefmap[*j] = *i;
		}
	}
}

string LegacyUpdateVisitor::GetValueForName(string name, strmap &prefmap)
{	strmap::iterator mi = prefmap.find(name);
	if(mi==prefmap.end())
		return string();
	else
		return mi->second;
}

// This is the function that implements stringicmp for the map
bool stringicmp::operator()(const string& first, const string& second) const 
{
	string firsti = first;
	string secondi = second;
	transform(firsti.begin(),firsti.end(), firsti.begin(), tolower);
	transform(secondi.begin(),secondi.end(), secondi.begin(), tolower);
	return(firsti < secondi);
}

///////////////////////////////////////////////////////////////////////////////////
// from website http://www.thecodezone.com/diary/archives/000057.html
///////////////////////////////////////////////////////////////////////////////////

StringTokenizer::StringTokenizer(const string &rStr, const string &rDelimiters)
{
	string::size_type lastPos(rStr.find_first_not_of(rDelimiters, 0));
	string::size_type pos(rStr.find_first_of(rDelimiters, lastPos));
	while (string::npos != pos || string::npos != lastPos)
	{
		push_back(rStr.substr(lastPos, pos - lastPos));
		lastPos = rStr.find_first_not_of(rDelimiters, pos);
		pos = rStr.find_first_of(rDelimiters, lastPos);
	}
}