#include <iostream>
#include <string>
#include <fstream>
#include <Windows.h>	// for DeleteFile(), MoveFile()

	const std::string OUT_FN_SUFFIX= ".tmp";
	const std::string filever1= "FILEVERSION";
	const std::string filever2= "VALUE \"FileVersion\"";
	const std::string prodver1= "PRODUCTVERSION";
	const std::string prodver2= "VALUE \"ProductVersion\"";
	int g_update= 0;

void printVer()
{
	std::cout << "Rvu.exe - Resouce Version Upgrader Utility" << " v1.0 " << std::endl;
	std::cout << "(C) 2004 Vanderbilt University" << std::endl;
	std::cout << "Institute for Software Integrated Systems" << std::endl << std::endl;
}

int usage()
{
	std::cout << "Usage: rvu.exe [rc_file] [file_ver] [product_ver]" << std::endl;
	std::cout << "rc_file: Windows resource file (.rc file)" << std::endl;
	std::cout << "file_ver: file version string" << std::endl;
	std::cout << "product_ver: product version string" << std::endl;
	std::cout << std::endl << "Example: rvu.exe component.rc 1.2.0 1.0.18" << std::endl;
	return -1;
}

int win_error( const std::string& err_msg)
{
	DWORD err= GetLastError();
	LPVOID lpMsgBuf;
	FormatMessage( 
	    FORMAT_MESSAGE_ALLOCATE_BUFFER | 
	    FORMAT_MESSAGE_FROM_SYSTEM | 
	    FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		err,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
		);
	std::cout << err_msg << (LPCTSTR)lpMsgBuf << std::endl;
	LocalFree( lpMsgBuf);
	return err;
}

// Trim leading white space characters.
std::string::difference_type trim( std::string& toTrim)
{
	std::string::iterator i;
	for( i= toTrim.begin(); isspace( *i); ++i) {}
	std::string::difference_type chars_trimmed= i- toTrim.begin();
	toTrim.erase( toTrim.begin(), i);
	return chars_trimmed;
}

// vertype: 0: -not found-, 1: FILEVERSION, 2: VALUE "FileVersion"
bool hasLineFileVer( const std::string& line, int& vertype, int& num_whitechars)
{
	std::string trimmed_line= line;
	num_whitechars= trim( trimmed_line);
	std::string::size_type find1= std::string::npos, find2= std::string::npos;
	if ( ( 0== ( find1= trimmed_line.find( filever1))) || 
		 ( 0== ( find2= trimmed_line.find( filever2)))) {
		vertype= (find2 == std::string::npos) ? 1 : 2;
		std::cout << "\t" << trimmed_line << " -> ";
		return true; // found
	}
	vertype= 0;
	return false;
}

// vertype: 0: -not found-, 1: PRODUCTVERSION, 2: VALUE "ProductVersion"
bool hasLineProductVer( const std::string& line, int& vertype, int& num_whitechars)
{
	std::string trimmed_line= line;
	num_whitechars= trim( trimmed_line);
	std::string::size_type find1 = std::string::npos, find2 = std::string::npos;
	if ( ( 0 == ( find1= trimmed_line.find( prodver1))) || 
		 ( 0 == ( find2= trimmed_line.find( prodver2)))) {
		vertype= (find2 == std::string::npos) ? 1 : 2;
		std::cout << "\t" << trimmed_line << " -> ";
		return true; // found
	}
	vertype= 0;
	return false;

}

void updateFileVer( std::string& line, const std::string& file_ver, int vertype, int num_whitechars)
{
	switch (vertype)
	{
	case 1:
		line= filever1+ " "+ file_ver;
		break;
	case 2:
		line= filever2+ ", \""+ file_ver+ "\\0\"";
		break;
	default:
		// do nothing
		return;
	}
	std::cout << line << std::endl;
	std::string trail_whitechars( num_whitechars, ' ');
	line= trail_whitechars+ line;
	++g_update;
}

void updateProductVer( std::string& line, const std::string& product_ver, int vertype, int num_whitechars)
{
	switch (vertype)
	{
	case 1:
		line= prodver1+ " "+ product_ver;
		break;
	case 2:
		line= prodver2+ ", \""+ product_ver+ "\\0\"";
		break;
	default:
		// do nothing
		return;
	}
	std::cout  << line << std::endl;
	std::string trail_whitechars( num_whitechars, ' ');
	line= trail_whitechars+ line;
	++g_update;
}

// All valid version numbers must start and end with a decimal number, 
// and can consist of max. 4 decimal numbers separated by commas.
int versionNumberParser( const std::string& version_number)
{
	bool prev_is_comma= true;	// must start with number
	std::string::const_iterator start= version_number.begin();
	std::string::const_iterator end= version_number.end();
	std::string::size_type size= version_number.size();
	int num_commas= 0;
	for ( std::string::const_iterator i= start; i!= end; ++ i) {
		if ( !isdigit( *i) && prev_is_comma)	// numbers separated by commas
			return i-start;
		if ( !isdigit( *i) && (*i != ','))	// only numbers and commas
			return i-start;
		if ( !isdigit( *i) && ( i== start+ size-1))	// must end with number
			return i-start;
		if ( !prev_is_comma && (*i == ','))	 { // update previous comma info for next iteration
			prev_is_comma= true;
			++num_commas;
			if ( 4 == num_commas)
				return i-start;		// max. 4 numbers
		}
		else
			prev_is_comma= false;
	}
	return size;	// parsed ok
}

// search for dot charcters in a string and replaces them with comma characters.
std::string convertDotsToCommas( const std::string& dotted)
{
	std::string converted= dotted;
	std::string::size_type dot_pos;
	while ( (dot_pos= converted.find( '.')) != std::string::npos)
	{
		converted[ dot_pos]= ',';
	}
	return converted;
}

void printDiagnostics( const std::string& parsed_str, int parse_err_location)
{
	if ( (unsigned int)parse_err_location < parsed_str.size()) {
		std::cout << "WARNING: Invalid version number format: " << std::endl;
		std::cout << parsed_str << std::endl;
		std::string trail_whitechars( parse_err_location, ' ');
		std::cout << trail_whitechars << "^" << std::endl;
		std::cout << "Valid version numbers must start and end with a decimal number and can consist of max. 4 decimal numbers separated by commas. Ex: \"1,2,3,0\"" << std::endl;
	} 
	else {
//		std::cout << "OK" << std::endl;
	}
}

void parseVerNum( const std::string& ver)
{
	/* 
	// test version numbers
	const std::string t1= "abcd";
	const std::string t2= "12,abcd";
	const std::string t3= ",123";
	const std::string t4= "12,";
	const std::string t5= "1,2,3,4,5";
	const std::string t6= "1,2,,3";
	const std::string t7= "1,2,3";	// OK
	printDiagnostics( t1, versionNumberParser( t1));
	printDiagnostics( t2, versionNumberParser( t2));
	printDiagnostics( t3, versionNumberParser( t3));
	printDiagnostics( t4, versionNumberParser( t4));
	printDiagnostics( t5, versionNumberParser( t5));
	printDiagnostics( t6, versionNumberParser( t6));
	printDiagnostics( t7, versionNumberParser( t7));
	*/
	printDiagnostics( ver, versionNumberParser( ver));
}

int main( int argc, char* argv[])
{
	printVer();
	if ( argc != 4)
		return usage();
	// do the upgrading	
	const char* rc_fn= argv[ 1];
	const std::string tmp_fn ( rc_fn+ OUT_FN_SUFFIX);
	std::string file_ver= convertDotsToCommas( argv[ 2]);
	parseVerNum( file_ver);
	std::string product_ver= convertDotsToCommas( argv[ 3]);
	parseVerNum( product_ver);
	std::ifstream is_rcfile( rc_fn);
	if ( is_rcfile.fail())
	{
		std::cout << "ERROR: Resource file '" << rc_fn << "' not found error." << std::endl << std::endl;
		return -1;
	}
	std::ofstream os_rcfile( tmp_fn.c_str());
	if ( os_rcfile.fail())
	{
		std::cout << "ERROR: Temporary file '" << tmp_fn << "' could not be opened for writing." << std::endl << std::endl;
		return -1;
	}
	std::string line;
	int vertype= 0;
	int num_whitechars= 0;
	std::cout << "Searching..." << std::endl;
	while ( !is_rcfile.eof())
	{
		std::getline( is_rcfile, line);
		if ( hasLineFileVer( line, vertype, num_whitechars))
			updateFileVer( line, file_ver, vertype, num_whitechars);
		else
			if ( hasLineProductVer( line, vertype, num_whitechars))
				updateProductVer( line, product_ver, vertype, num_whitechars);
		os_rcfile << line << std::endl;
	}
	// if successful, delete the old resource, rename the new resource
	is_rcfile.close();
	os_rcfile.close();
	if ( !DeleteFile( rc_fn)) {
		return win_error( std::string( rc_fn) + " delete failed. ");
	}
	if ( !MoveFile( tmp_fn.c_str(), rc_fn)) {
		return win_error( std::string( "Renaming ") + tmp_fn + " to "+ rc_fn + " failed. ");
	}
	std::cout << rc_fn; 
	std::cout << " has been successfully upgraded (in " << g_update << " lines)." << std::endl;
	return 0;
	
}